import org.apache.commons.codec.binary.Base64
import org.apache.commons.io.IOUtils
import org.apache.log4j.Logger

zephyr = new ZephyrTool(restUserName: restUserName, restPassword: restPassword, baseUrl: baseurl)

/**
 * Velocity tool to get Zephyr test data from ZAPI (Zephyr REST API).
 * Note: you must install the ZAPI app (https://marketplace.atlassian.com/plugins/com.thed.zephyr.zapi/server/overview) for this.
 *
 * @see https://zephyrdocs.atlassian.net/wiki/spaces/DEVELOPER/pages/33095703/ZAPI+Server
 */
class ZephyrTool {
	def log = Logger.getLogger(this.getClass())

	/* Zephyr REST API request parameters. */
	String baseUrl
	String restUserName
	String restPassword

	def slurper = new groovy.json.JsonSlurper()

	/**
	 * Returns the Zephyr test steps for the passed issue (as Groovy objects slurped from <code>http://docs.getzephyr.apiary.io/#reference/teststepresource/get-update-delete-teststep/get-teststep<code>)
	 * or the exception object if something went wrong.
	 */
	def getTestSteps(def issueId) {
		log.debug("Downloading Zephyr test steps for issue #${issueId}")
		def testSteps = callZapi("${baseUrl}/rest/zapi/latest/teststep/${issueId}")
		if(!(testSteps instanceof Throwable)) {
			log.debug("${testSteps.size()} Zephyr test steps found for issue #${issueId}")
		}
		return testSteps
	}

	/**
	 * Returns the Zephyr test executions for the passed issue (as Groovy objects slurped from <code>http://docs.getzephyr.apiary.io/#reference/executionresource/get-list-of-execution/get-list-of-execution<code>)
	 * or the exception object if something went wrong.
	 */
	def getTestExecutions(def issueId) {
		log.debug("Downloading Zephyr test executions for issue #${issueId}")
		def testExecutions = callZapi("${baseUrl}/rest/zapi/latest/execution?issueId=${issueId}")
		if(!(testExecutions instanceof Throwable)) {
			log.debug("${testExecutions.executions.size()} Zephyr test executions found for issue #${issueId}")
		}
		return testExecutions
	}

	/**
	 * Returns the Zephyr test step results for the passed execution (as Groovy objects slurped from <code>http://docs.getzephyr.apiary.io/#reference/stepresultresource/get-list-of-step-result/get-list-of-step-result<code>)
	 * or the exception object if something went wrong.
	 */
	def getTestStepResults(def executionId) {
		log.debug("Downloading Zephyr test step results for execution #${executionId}")
		def testStepResults = callZapi("${baseUrl}/rest/zapi/latest/stepResult?executionId=${executionId}")
		if(!(testStepResults instanceof Throwable)) {
			log.debug("${testStepResults.size()} Zephyr test step results found for execution #${executionId}")
		}
		return testStepResults
	}

	def callZapi(String url) {
		HttpURLConnection connection
		InputStream inputStream

		try {
			log.debug("Zephyr URL: ${url}")
			connection = new URL(url).openConnection()
			String authValue = "${restUserName}:${restPassword}"
			connection.setRequestProperty("Authorization", "Basic " + Base64.encodeBase64String(authValue.getBytes("ISO-8859-1")))
			inputStream = connection.getInputStream()
			def json = IOUtils.toString(inputStream, "UTF-8")
			log.trace("Zephyr JSON for ${url}:\n${json}")

			def result = slurper.parseText(json)

			return result
		} catch (Throwable e) {
			log.error("Failed to download Zephyr test data", e)
			return e
		} finally {
			connection.disconnect()
			IOUtils.closeQuietly(inputStream)
		}
	}
}
